@@ -8,7 +8,7 @@ use base qw(DBIx::Class);
 
 use version;
 use vars qw($VERSION);
-$VERSION = version->new("1.03");
+$VERSION = version->new("1.04");
 
 =encoding utf8
 
@@ -72,7 +72,7 @@ DBIx::Class::DeleteAction - Define delete triggers
     $actor->delete();    
  });
  # Deletes all related actorroles only if they don't have a production
- # Finally deletes the actor itself
+ # Finally deletes the actor itself (Always use transactions!!!)
  
  $schema->txn_do(sub {
     $actor_role->delete();    
@@ -114,13 +114,13 @@ Deny deletion if this record is being referenced from other rows.
 
 =item * CODE reference
 
-Executes the code referece on delete. The current C<DBIx::Class::Row> object 
+Executes the code referece on delete. The current L<DBIx::Class::Row> object 
 and the name of the relation are passed to the code reference.
 
 =item * STRING
 
 Execute a method with the given name. The method will be called on the current
-C<DBIx::Class::Row> object and will be passed the name of the relation.
+L<DBIx::Class::Row> object and will be passed the name of the relation.
 
 =item * ignore
 
@@ -181,11 +181,11 @@ This method overdrives the L<DBIx::Class::Row> delete method. You can
 add arbitrary data as HASHREF which will be passed to your custom
 delete handles. 
 
-Make sure that you ALWAYS call C<delete> always from within a TRANSACTION 
+Make sure that you ALWAYS call C<delete> always inside a TRANSACTION 
 block. 
 
-If you call C<delete> from within a custom delete handler always pass on
-the C<seen> parameter.
+If you call another C<delete> from within a custom delete handler always pass 
+on the C<seen> parameter.
 
 =cut
 
@@ -209,8 +209,7 @@ sub delete {
     my $debug = $self->result_source->storage->debug();
     
     my $identifier = $self->_delete_action_identifier;
-    
-    
+       
     # Check for identifier
     return if (grep { $identifier eq $_ } @{$params->{seen}});
     
@@ -245,7 +244,7 @@ sub delete {
         if ($relationship_info->{attrs}{accessor} eq 'multi') {
             $related = $self->search_related($relationship);
             next RELATIONSHIP
-                unless $related->count;
+                unless $related;
         } else {
             CONDITIONS:
             foreach my $condition (values %{$relationship_info->{cond}}) {
@@ -253,7 +252,6 @@ sub delete {
                     my $column = $1;
                     next RELATIONSHIP
                         unless $self->get_column($column);
-#                    
 #                    unless ($self->has_column_loaded($column)) {
 #                        warn("LOAD $column");
 #                        $self = $self->get_from_storage();
@@ -262,7 +260,6 @@ sub delete {
 #                        last CONDITIONS;
 #                    }
                 }
-                
             }
 
             $related = $self->$relationship;
@@ -303,40 +300,47 @@ sub delete {
             } else {
                 warn("Delete action 'null' does not work with ".$relationship_info->{attrs}{accessor}." relations");
             }
-        # Action: DENY
-        } elsif ($delete_action eq 'deny') {
-            warn('DeleteAction: DENY '.$self.'->'.$relationship) if $debug;
-            if ($related->isa('DBIx::Class::ResultSet')) {
-                while (my $item = $related->next) {
-                    my $compare_identifier = $item->_delete_action_identifier;
-                    next if grep {$compare_identifier eq $_} @{$params->{seen}};
-                    $self->throw_exception("Can't delete the object because it is still referenced from other records");
-                }
-            } else {
-                my $compare_identifier = $related->_delete_action_identifier;
-                unless (grep {$compare_identifier eq $_} @{$params->{seen}}) {
-                    $self->throw_exception("Can't delete the object because it is still referenced from other records");
-                }
-            }
-        # Action: CODE
-        } elsif (ref $delete_action eq 'CODE') {
-            warn('DeleteAction: CODE '.$self.'->'.$relationship) if $debug;
-            $delete_action->($self,{
-                relationship    => $relationship,
-                related         => $related,
-                %{$params},
-            });
-        # Action: METHOD    
-        } elsif ($self->can($delete_action)) {
-            warn('DeleteAction: METHOD '.$self.'->'.$relationship.':'.$delete_action) if $debug;
-            $self->$delete_action({
-                relationship    => $relationship,
-                related         => $related,
-                %{$params},
-            });
-        # Fallback
         } else {
-            $self->throw_exception("Invalid delete action '$delete_action'")
+            if ($related->isa('DBIx::Class::ResultSet')
+                && $related->count == 0) {
+                next RELATIONSHIP;
+            }    
+            
+            # Action: DENY
+            if ($delete_action eq 'deny') {
+                warn('DeleteAction: DENY '.$self.'->'.$relationship) if $debug;
+                if ($related->isa('DBIx::Class::ResultSet')) {
+                    while (my $item = $related->next) {
+                        my $compare_identifier = $item->_delete_action_identifier;
+                        next if grep {$compare_identifier eq $_} @{$params->{seen}};
+                        $self->throw_exception("Can't delete the object because it is still referenced from other records");
+                    }
+                } else {
+                    my $compare_identifier = $related->_delete_action_identifier;
+                    unless (grep {$compare_identifier eq $_} @{$params->{seen}}) {
+                        $self->throw_exception("Can't delete the object because it is still referenced from other records");
+                    }
+                }
+            # Action: CODE
+            } elsif (ref $delete_action eq 'CODE') {
+                warn('DeleteAction: CODE '.$self.'->'.$relationship) if $debug;
+                $delete_action->($self,{
+                    relationship    => $relationship,
+                    related         => $related,
+                    %{$params},
+                });
+            # Action: METHOD    
+            } elsif ($self->can($delete_action)) {
+                warn('DeleteAction: METHOD '.$self.'->'.$relationship.':'.$delete_action) if $debug;
+                $self->$delete_action({
+                    relationship    => $relationship,
+                    related         => $related,
+                    %{$params},
+                });
+            # Fallback
+            } else {
+                $self->throw_exception("Invalid delete action '$delete_action'")
+            }
         }
     }
 
@@ -346,21 +350,68 @@ sub delete {
     return $self->next::method($params);
 }
 
+=head1 EXAMPLE
+
+=head2 Tree example
+
+This example shows a very simple tree schema, where each node points to its
+parent node. Once you delete an item from the tree, all child nodes will be
+appended to the parent node of the deleted node.
+
+ package MyApp::Treenode;
+ use strict;
+ use warnings;
+ 
+ use parent qw(DBIx::Class);
+ 
+ __PACKAGE__->load_components(
+   "+DBIx::Class::DeleteAction",
+   "PK",
+   "Core",
+ );
+ 
+ __PACKAGE__->table("treenode");
+ __PACKAGE__->add_columns(qw/id name parent/);
+ __PACKAGE__->set_primary_key("id");
+ 
+ # Do not delete parent node
+ __PACKAGE__->might_have(
+    'parent' => 'MyApp::Treenode',
+    { "foreign.id" => "self.parent" },
+    { delete_action => 'ignore' },
+ );
+ 
+ # Update all child nodes
+ __PACKAGE__->has_many(
+    'children' => 'MyApp::Treenode',
+    { "foreign.parent" => "self.id" },
+    { delete_action => sub {
+        my ($self,$params) = @_;
+        $params->{related}->update({
+            parent  => $self->get_column('parent'), 
+        });
+    } },
+ );
+
+=head2 Debugging
+
+Use C<DBIC_TRACE=1> or set C<__PACKAGE__->storage->debug(1);> to see what
+is exactly going on.
 
 =head1 CAVEATS
 
-Note that the C<delete> method in C<DBIx::Class::ResultSet> will not run 
+Note that the C<delete> method in L<DBIx::Class::ResultSet> will not run 
 DeleteAction triggers. See C<delete_all> if you need triggers to run.
 
-Any database-level cascade, restrict or trigger will be performed AFTER a 
-DBIx-Class-DeleteAction based trigger.
+Any database-level cascade, restrict or trigger will be performed AFTER 
+DBIx-Class-DeleteAction based triggers.
 
 Always use transactions, or else you might end up with inconsistent data.
 
 =head1 SUPPORT
 
 Please report any bugs or feature requests to 
-C<bug-dbix-class-deleteaction@rt.cpan.org>, or through the web interface at
+L<bug-dbix-class-deleteaction@rt.cpan.org>, or through the web interface at
 L<http://rt.cpan.org/Public/Bug/Report.html?Queue=DBIx::Class::DeleteAction>.
 I will be notified, and then you'll automatically be notified of progress on 
 your report as I make changes.
@@ -379,7 +430,7 @@ software company I run with Koki and Domm (L<http://search.cpan.org/~domm/>).
 
 =head1 COPYRIGHT
 
-DBIx::Class::DeleteAction is Copyright (c) 2008 Maroš Kollár 
+DBIx::Class::DeleteAction is Copyright (c) 2008-9 Maroš Kollár 
 - L<http://www.revdev.at>
 
 This program is free software; you can redistribute it and/or modify it under 
@@ -390,4 +441,4 @@ LICENSE file included with this module.
 
 =cut
 
-"Delete me NAAAT";
\ No newline at end of file
+"Delete me .... NAAAT";
\ No newline at end of file
