@@ -27,6 +27,12 @@ __PACKAGE__->has_many(
     }
 );
 
-
+__PACKAGE__->has_many(
+    'ds' => 'DATest::Schema::Test2D', 
+    { 'foreign.c'  => 'self.id' },
+    { 
+        delete_action   => 'ignore',
+    }
+);
    
 1;
\ No newline at end of file
